/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WMSHCRACK_H
#define WMSHCRACK_H

#include "tbox.h"
#include "wObject.h"
#include "wMshExport.h"
#include <vector>
#include <map>
#include <memory>

namespace tbox
{

/**
 * @brief Duplicate nodes and elements in a physical group and make mesh consistent
 *
 * Similar to plugin "crack" from gmsh but allows more control. Currently coded
 * only for Line2 and Tri3.
 * @authors Adrien Crovato
 */
class TBOX_API MshCrack : public fwk::wSharedObject
{
private:
    std::shared_ptr<MshData> msh;    ///< mesh
    int nDim;                        ///< number of dimensions
    bool hasCrack;                   ///< flag telling if mesh already has crack
    std::map<Node *, Node *> nodMap; ///< map between nodes on both side of crack
    Group *crckGrp;                  ///< physical group of crack
    Group *exclGrp;                  ///< physical group of nodes not to be duplicated
    std::vector<Groups *> bndGrps;   ///< list of pair of physical groups touching crack
    std::vector<Group *> bndGrp;     ///< list of physical groups touching crack

    void openCrack();
    void modifyBoundaries_();
    void modifyBoundaries();
    void swapNodes(Element *e);

public:
    MshCrack(std::shared_ptr<MshData> _msh, int _nDim);
    virtual ~MshCrack();

    void setCrack(std::string const &crckName);
    void addBoundaries(std::vector<std::string> const &bndName);
    void setExcluded(std::string const &exclName);
    void run();

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace tbox

#endif //WMSHCRACK_H